==> okapi-ir-0.2.3-sources.jar: org/opencypher/okapi/ir/api/pattern/Endpoints.scala <==
/*
 * Copyright (c) 2016-2018 "Neo4j Sweden, AB" [https://neo4j.com]
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Attribution Notice under the terms of the Apache License 2.0
 *
 * This work was created by the collective efforts of the openCypher community.
 * Without limiting the terms of Section 6, any Derivative Work that is not
 * approved by the public consensus process of the openCypher Implementers Group
 * should not be described as “Cypher” (and Cypher® is a registered trademark of
 * Neo4j Inc.) or as "openCypher". Extensions by implementers or prototypes or
 * proposals for change that have been documented or implemented should only be
 * described as "implementation extensions to Cypher" or as "proposed changes to
 * Cypher that are not yet approved by the openCypher community".
 */
package org.opencypher.okapi.ir.api.pattern

import org.opencypher.okapi.ir.api.IRField

import scala.language.implicitConversions

sealed trait Endpoints extends Traversable[IRField] {
  def contains(f: IRField): Boolean
}

case object Endpoints {

  def apply(source: IRField, target: IRField): Endpoints =
    if (source == target) OneSingleEndpoint(source) else TwoDifferentEndpoints(source, target)

  implicit def one(field: IRField): IdenticalEndpoints =
    OneSingleEndpoint(field)

  implicit def two(fields: (IRField, IRField)): Endpoints = {
    val (source, target) = fields
    apply(source, target)
  }

  private case class OneSingleEndpoint(field: IRField) extends IdenticalEndpoints {
    override def contains(f: IRField): Boolean = field == f
  }
  private case class TwoDifferentEndpoints(source: IRField, target: IRField) extends DifferentEndpoints {
    override def flip: TwoDifferentEndpoints = copy(target, source)

    override def contains(f: IRField): Boolean = f == source || f == target
  }
}

sealed trait IdenticalEndpoints extends Endpoints {
  def field: IRField

  final override def foreach[U](f: IRField => U): Unit = f(field)
}

sealed trait DifferentEndpoints extends Endpoints {
  def source: IRField
  def target: IRField

  def flip: DifferentEndpoints

  override def foreach[U](f: IRField => U): Unit = { f(source); f(target) }
}

==> okapi-ir-0.3.2-sources.jar: org/opencypher/okapi/ir/api/pattern/Endpoints.scala <==
/*
 * Copyright (c) 2016-2019 "Neo4j Sweden, AB" [https://neo4j.com]
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Attribution Notice under the terms of the Apache License 2.0
 *
 * This work was created by the collective efforts of the openCypher community.
 * Without limiting the terms of Section 6, any Derivative Work that is not
 * approved by the public consensus process of the openCypher Implementers Group
 * should not be described as “Cypher” (and Cypher® is a registered trademark of
 * Neo4j Inc.) or as "openCypher". Extensions by implementers or prototypes or
 * proposals for change that have been documented or implemented should only be
 * described as "implementation extensions to Cypher" or as "proposed changes to
 * Cypher that are not yet approved by the openCypher community".
 */
package org.opencypher.okapi.ir.api.pattern

import org.opencypher.okapi.ir.api.IRField

import scala.language.implicitConversions

sealed trait Endpoints extends Traversable[IRField] {
  def contains(f: IRField): Boolean
}

case object Endpoints {

  def apply(source: IRField, target: IRField): Endpoints =
    if (source == target) OneSingleEndpoint(source) else TwoDifferentEndpoints(source, target)

  implicit def one(field: IRField): IdenticalEndpoints =
    OneSingleEndpoint(field)

  implicit def two(fields: (IRField, IRField)): DifferentEndpoints = {
    val (source, target) = fields
    TwoDifferentEndpoints(source, target)
  }

  private case class OneSingleEndpoint(field: IRField) extends IdenticalEndpoints {
    override def contains(f: IRField): Boolean = field == f
  }
  private case class TwoDifferentEndpoints(source: IRField, target: IRField) extends DifferentEndpoints {
    override def flip: TwoDifferentEndpoints = copy(target, source)

    override def contains(f: IRField): Boolean = f == source || f == target
  }
}

sealed trait IdenticalEndpoints extends Endpoints {
  def field: IRField

  final override def foreach[U](f: IRField => U): Unit = f(field)
}

sealed trait DifferentEndpoints extends Endpoints {
  def source: IRField
  def target: IRField

  def flip: DifferentEndpoints

  override def foreach[U](f: IRField => U): Unit = { f(source); f(target) }
}
